#
# Licensed Materials - Property of IBM
# Rational ClearQuest
# (C) Copyright IBM Corp. 2008, 2011.  All Rights Reserved
# US Government Users Restricted Rights
# Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
#

import os, sys, java, string


#-----------------------------------------------------------------
# Check to see if we are running under Windows or Unix
#-----------------------------------------------------------------
def isunix():
    os_name = java.lang.System.getProperty('os.name')
    os_name = os_name.lower()
    print "OS: " + os_name
    return os_name.find('windows') < 0


#-----------------------------------------------------------------
# On Unix, we need to set CQ environment under WAS
#-----------------------------------------------------------------
def add_cq_envs_to_was_server():

    if not isunix():
        print "not unix skipping env set"
        return 

    print "Setting WAS CQ envs"

    clearquest = r"/opt/rational/clearquest/bin/clearquest"
    tempfile = r".cq-output-tempfile.txt"

    print "ClearQuest: " + clearquest

    command = '"%s" -dumpsh > "%s"' % (clearquest, tempfile) 

    os.system(command)

    input = open(tempfile, 'r')
    Lines = input.readlines()
    input.close()

    cqenvs = {}

    for l in Lines:
        # skip over export lines.  Don't care about them
        if l.startswith("export"): continue
        parts = l.split("=")
        value = parts[1]
        # strip off the begining "'" and the trailing "';\n"
        value = value[1:-3]
        cqenvs[ parts[0] ] = value

    os.remove(tempfile)

    serverid = AdminConfig.getid('/Server:server1/JavaProcessDef:/')
    server = AdminConfig.list('JavaProcessDef', serverid)

    config_was_env = AdminConfig.showAttribute(server, 'environment')

    config_was_env = config_was_env[1:-1]
    existing_was_envs = config_was_env.split(' ')

    # if env entries already exist then update them
    for was_env in existing_was_envs:
        if len(was_env) == 0: continue
        name = AdminConfig.showAttribute(was_env, 'name')
        if cqenvs.has_key(name):
            print "Modify: " + name
            AdminConfig.modify(was_env, [['value', cqenvs[ name ]]])
            del cqenvs[ name ]

    # add any missing envs
    for key in cqenvs.keys():
        print "Add: " + key
        value = [ ['name', key], ['value', cqenvs[key]] ]
        AdminConfig.create('Property', server, value, 'environment') 

    AdminConfig.save()


#-----------------------------------------------------------------
# Given a property string, return it as a list array
#-----------------------------------------------------------------
def splitProps(propStr):
    propList = []
    tempProp = ''
    inQuotes = 0
    for char in propStr[1:-1]:
        if char == ' ':
            if inQuotes == 0:
                propList.append(tempProp)
                tempProp = ''
            else:
                tempProp = tempProp + char
        elif char == '"':
            if inQuotes == 0:
                inQuotes = 1
            else:
                inQuotes = 0
        else:
            tempProp = tempProp + char
    if tempProp != '':
        propList.append(tempProp)
    return propList
    

#-----------------------------------------------------------------
# Sets JVM's variable for solr.solr.home
#-----------------------------------------------------------------
def setSolrHome(interactive, solrHome):

    global AdminApp
    global AdminConfig

    server1 = AdminConfig.getid('/Server:server1/JavaProcessDef:/JavaVirtualMachine:/')
    jvm = AdminConfig.list('JavaVirtualMachine', server1)

    # NOTE: So that we are backward compatible, pre-CQ 8.0.0.1, we must always check if
    # an old setting for 'solr.solr.home' at the old location 'genericJvmArguments' 
    # exit, if so, remove it.
    existingGenericJvmArguments = AdminConfig.showAttribute(jvm, 'genericJvmArguments')
    items = string.split(existingGenericJvmArguments, " ")

    # Walk through existing items and if you find 'solr.solr.home' eliminate it from the list
    if len(items) >= 1:
        newGenericJvmArguments = ""
        removeOldJvmArg = 0
        for item in items:
            keyVal = string.split(item, "=")
            if len(keyVal) == 2:
                if keyVal[0] == "-Dsolr.solr.home":
                    removeOldJvmArg = 1
                    continue    # skip over 'solr.solr.home' since we want to remove it
            newGenericJvmArguments += " " + item

        if removeOldJvmArg:
            print "Removing pre-CQ 8.0.0.1 'solr.solr.home' from 'genericJvmArguments'"
            print "Current 'genericJvmArguments is: '" + newGenericJvmArguments + "'"
            AdminConfig.modify(jvm, [['genericJvmArguments', newGenericJvmArguments]])

    # Get back the current value for 'solr.solr.home'
    existingGenericJvmArguments = ""
    prop = []
    props = AdminConfig.showAttribute(jvm, 'systemProperties')
    for prop in splitProps(props)[:]:
        if AdminConfig.showAttribute(prop, 'name') == "solr.solr.home":
            existingGenericJvmArguments = AdminConfig.showAttribute(prop, 'value')
            break

    print ""
    print "This script change JVM's 'systemProperties' to set 'solr.solr.home'"
    print ""
    print "Current setting for 'solr.solr.home' is: '" + existingGenericJvmArguments + "'"

    # Since there is no 'modify' on an existing JVM property, we frst 
    # remove it (if one exist) and then add the new one (if one is given)
    prop = []
    props = AdminConfig.showAttribute(jvm, 'systemProperties')
    for prop in splitProps(props)[:]:
        if AdminConfig.showAttribute(prop, 'name') == "solr.solr.home":
            AdminConfig.remove(prop)

    # If we are given a new 'solrHome', we will set it
    if solrHome != "":
        attrs = [['name', "solr.solr.home"], ['value', solrHome]]
        AdminConfig.create('Property', jvm, attrs, 'systemProperties')

    AdminConfig.save()

    print ""
    print "'solr.solr.home' was set to: '" + solrHome + "'"
    print ""
    print "NOTE: CQ FTS WAS Profile MUST be re-started for this change to take effect."
    print ""


#-----------------------------------------------------------------
# Main
#-----------------------------------------------------------------
if (len(sys.argv) != 2):
    print "This script will change the java system property to include 'solr.solr.home'"
    print "In doing so, it will change the value of 'genericJvmArguments' in your server.xml file"
    print ""
    print "This script requires 2 parameter: "
    print "	  1) -interactive:[yes | no]"
    print "	  2) the full path to what solr.solr.home should be set to"
    print ""
    print "e.g.:     setSolrHome -interactive:yes /CQ.Solr/SAMPL/Solr/solr"
    print ""
    print "NOTE: '-interactive:[yes | no]' is depreciated as of CQ 8.0.0.1"
    print "      If we remove it from the script, we will break IM"
else:
    add_cq_envs_to_was_server()
    setSolrHome(sys.argv[0], sys.argv[1])
